macro_rules! assert_pred {
($($content:tt)*) => { ... };
}Expand description
Asserts that the given predicate applied to the given arguments returns true, panicking if it does not.
Note for users of GoogleTest for C++:
This differs from the ASSERT_PRED* family of macros in that it panics
rather than triggering an early return from the invoking function. To get
behaviour equivalent to ASSERT_PRED*, use verify_pred! with the ?
operator.