Module matcher

Module matcher 

The components required to implement matchers.

Enums

MatcherResult
The result of applying a Matcher on an actual value.

Traits

Describable
An item, normally a Matcher with positive and negative valences which can be turned into a Description for human consumption.
Matcher
Represents an arbitrary condition on data of the given type which can be checked to perform an assertion.
MatcherExt
Extension methods for composing matchers.