MatcherResult

Enum MatcherResult 

pub enum MatcherResult {
    Match,
    NoMatch,
}
The result of applying a Matcher on an actual value.

Variants



Match

The actual value matches according to the Matcher definition.



NoMatch

The actual value does not match according to the Matcher definition.

Implementations

Source

impl MatcherResult

pub fn is_match(self) -> bool

Returns true if self is MatcherResult::Match, otherwise false.

pub fn is_no_match(self) -> bool

Returns true if self is MatcherResult::NoMatch, otherwise false.