ContainerContainsUnorderedMatcher

Struct ContainerContainsUnorderedMatcher 

pub struct ContainerContainsUnorderedMatcher<'matchers, ContainerT: ?Sized, T: Debug, ModeT> { /* private fields */ }
Matches elements of an iterable collection with a sequence of matchers, in any order.

This struct is meant to be used only through the contains_exactly![...] macro.

Implementations

Source

impl<'matchers, ContainerT: ?Sized, T: Debug, ModeT> ContainerContainsUnorderedMatcher<'matchers, ContainerT, T, ModeT>

pub fn in_order( self, ) -> ContainerContainsOrderedMatcher<'matchers, ContainerT, T, ModeT>

Asserts that the matchers must match the elements of the collection in the same order as they appear when iterating.