DisplayMatcher

Struct DisplayMatcher 

pub struct DisplayMatcher<InnerMatcher> { /* private fields */ }
A matcher which renders the actual value as a String with Display and matches the result against the given inner matcher.

Implementations

Source

impl<InnerMatcher> DisplayMatcher<InnerMatcher>

pub fn alternate(self) -> Self

Indicates that the match should be made against the alternate rendering of the actual value. That is the result of formatting it with {:#}.