pub struct NearMatcher<T: Debug> { /* private fields */ }Available on crate feature
num-traits only.Expand description
A matcher which matches floating-point numbers approximately equal to its expected value.
Implementations
Sourceimpl<T: Debug> NearMatcher<T>
impl<T: Debug> NearMatcher<T>
Sourcepub fn nans_are_equal(self) -> Self
pub fn nans_are_equal(self) -> Self
Configures this instance to treat two NaNs as equal.
This behaviour differs from the IEEE standad for floating point which treats two NaNs as infinitely far apart.
Sourcepub fn nans_are_not_equal(self) -> Self
pub fn nans_are_not_equal(self) -> Self
Configures this instance to treat two NaNs as not equal.
This behaviour complies with the IEEE standad for floating point. It is the default behaviour for this matcher, so invoking this method is usually redunant.