Module result

Module result 

Traits

OrFailExt
Provides an extension method for converting an arbitrary type into a Result.
TestResultExt
Adds to Result support for Test That! functionality.

Functions

verify_current_test_outcomestd
Returns a Result corresponding to the outcome of the currently running test.

Type Aliases

ResultDeprecatedgoogletest-compat
Alias for TestResult to ease porting from googletest.
TestResult
A Result whose Err variant indicates a test failure.