The components required to implement matchers.
Enums
- Matcher
Result - The result of applying a
Matcheron an actual value.
Traits
- Describable
- An item, normally a Matcher with positive and negative valences which can be turned into a Description for human consumption.
- Matcher
- Represents an arbitrary condition on data of the given type which can be checked to perform an assertion.
- Matcher
Ext - Extension methods for composing matchers.