pub struct ContainerContainsUnorderedMatcher<'matchers, ContainerT: ?Sized, T: Debug, ModeT> { /* private fields */ }Expand description
Matches elements of an iterable collection with a sequence of matchers, in any order.
This struct is meant to be used only through the
contains_exactly![...] macro.
Implementations
Sourceimpl<'matchers, ContainerT: ?Sized, T: Debug, ModeT> ContainerContainsUnorderedMatcher<'matchers, ContainerT, T, ModeT>
impl<'matchers, ContainerT: ?Sized, T: Debug, ModeT> ContainerContainsUnorderedMatcher<'matchers, ContainerT, T, ModeT>