Traits
- OrFail
Ext - Provides an extension method for converting an arbitrary type into a
Result. - Test
Result Ext - Adds to
Resultsupport for Test That! functionality.
Functions
- verify_
current_ test_ outcome std - Returns a
Resultcorresponding to the outcome of the currently running test.
Type Aliases
- Result
Deprecated googletest-compat - Alias for TestResult to ease porting from googletest.
- Test
Result - A
ResultwhoseErrvariant indicates a test failure.